#include <detect.h>
#include <math.h>

/*----------------------------------------------------------------------*
 *	Function: VOID Onset(ibak,sper,tm_indx,				*
 * 			tc,amp_indx,pt0,pt1,pt2,pt3,pt4,                *
 *                       detPtr, onset_Ptr, onsetN, onsetM)             *
 *					-Calculate onset parameters	*
 *----------------------------------------------------------------------*
 *	Arguments:							*
 *			LONG ibak    - # of P-T values looked back	*
 *			LONG sper    - period of signal (samples)	*
 *			LONG tm_indx - index to reference time of	*
 *					 first P-T value of signal	*
 *			LONG tc      - time correction			*
 *			LONG amp_indx- index to first P-T value of	*
 *					  the signal			*
 *			LONG pt0     - value 1 for SNR calc		*
 *			LONG pt1     - value 2 for SNR calc		*
 *			LONG pt2     - value 3 for SNR calc		*
 *			LONG pt3     - value 4 for SNR calc		*
 *			LONG pt4     - value 5 for SNR calc		*
 *----------------------------------------------------------------------*
 *	Returns:	Nothing (VOID)					*
 *----------------------------------------------------------------------*
 *	Fatal Errors:	This routine does not have fatal exits		*
 *----------------------------------------------------------------------*
 *	Calculates period, amplitude,SNR series (O-F 83-785), and 	*
 *	converts sample number to time. Outputs the estimates.		*
 *----------------------------------------------------------------------*/

VOID Onset(ibak, sper, tm_indx, tc, amp_indx, pt0,
	pt1, pt2, pt3, pt4,
          detector, oPtr, nOnsetPtr, maxOnset)

LONG ibak, sper, tm_indx, amp_indx;
LONG pt0, pt1, pt2, pt3, pt4, tc;
struct detect_info *detector;
struct s_onset *oPtr;
int * nOnsetPtr;

{
        LONG i, indx2, j, isnr[5];
	LONG mx_amp, base, pt[5], temp, onset1;
	LONG t_samp, d_samp;
	TEXT ipol;
        struct s_onset onsetdata;
        FILE *onFilePtr;
        int nOnset = *nOnsetPtr;
        double maxAmp;
        double freq;
        int ms;    
      	int frIndex[3];
	int loopLim[3];

        frIndex[0] = 128;
        frIndex[1] = 256;
        frIndex[2] = 1024;

	loopLim[0] =  8;
	loopLim[1] = 10;
	loopLim[2] = 20;
   
/*---------------get polarity of first break of the signal--------------*/

	if (pt2 > 0)
	    ipol = 'd';		       /* pt2 is first P-T of signal */
	else
	    ipol = 'c';

/*-----------------------load array for SNR calculation------------------*/
/*------------------------see OF Report 83-785---------------------------*/
	pt[0] = pt0;		       /* 2nd P-T before signal */
	pt[1] = pt1;		       /* P-T immediately before signal */
	pt[2] = pt2;		       /* 1st P-T of signal */
	pt[3] = pt3;		       /* 2nd P-T of signal */
	pt[4] = pt4;		       /* 3rd P-T of signal */

/*------------------do the SNR calculation without using long division----*/

	for (j = 0; j < 5; j++) {
	    base = con_ptr->twosd - (con_ptr->twosd >> 1);
	    for (i = 0; i < 10; i++) {
	        isnr[j] = i;
	        if (absval(pt[j]) < base)
	    	break;
	        base += con_ptr->twosd;
	    }
	}

/*---------get the maximum amplitude of the first <loopLim> PTs -------------*/
      

	mx_amp = 0;
	for (i=0; i<3; i++){
          if (detector->sam_sec == frIndex[i]){
            for (j = 0; j < loopLim[i]; j++) {
	      indx2 = amp_indx++;
	      temp = absval(con_ptr->buf_amp[indx2]);
	      if (temp > mx_amp){
	        mx_amp = temp;
	      }
	      if (indx2++ == B_M1){
	        indx2 = 0;		       /* indx2 incremented */
	      }
	    }
	  }
	}

/*------------calculate the Period(in seconds) of the first 4 cycles----*/
  
	Period(sper);

/*--------Compute which record the data came from so that the time will------*/
/*--------be calculated correctly.  -----------------------------------------*/

	t_samp = con_ptr->buf_tim[tm_indx];
	d_samp = con_ptr->cur_rec - con_ptr->buf_rec[tm_indx];
	if (d_samp < 0)
	    d_samp += CUR_MAX;

	if (d_samp > 0)
	    t_samp -= con_ptr->sam_ch * d_samp;

/*---------set up the onset time, amplitude and frequency  for output----- -*/

	onset1 = Time_f(t_samp, tc);
        maxAmp = (double)(mx_amp)/(double)(con_ptr->twosd);      
        freq = 1./((double)(p_lval) + (double)(p_rval)*pow(10,-9));
	/*	printf("twosd %d, mx_amp %d, maxAmp %f\n", con_ptr->twosd,mx_amp,maxAmp); */
/*------------This code is for writing data into output records-------------*/
	onsetdata.new_onset = TRUE;	       /* New onset has occured */
/*	onsetdata.o_polar = ipol;                         */
/*	onsetdata.o_dalgo = con_ptr->iset ? 'A' : 'B';    */ 
	for (i = 0; i < 5; i++)
            onsetdata.o_snr[i] = isnr[i];
	onsetdata.o_secs = onset1;
	onsetdata.o_ns = t_rval;
	onsetdata.o_amps = mx_amp;
	onsetdata.o_pl = p_lval;
	onsetdata.o_pr = p_rval;
	onsetdata.o_large = con_ptr->twosd;
        onsetdata.o_maxamp = maxAmp;
        onsetdata.o_freq = freq; 

        ms = p_rval/1000000;

      if (nOnset < maxOnset){
        memcpy((oPtr + nOnset), &onsetdata, sizeof(struct s_onset));
          nOnset++;
      }
      *nOnsetPtr = nOnset;

/*--------etime will be changed when events are detected - backup--------*/

	etime.sec = htime.sec;
	etime.ns = htime.ns;

/*----The thresholds are raised to permit processing the coda of events, --*/

#ifdef LOCAL_EV                                                      
	con_ptr->twosd = mx_amp;                                      
   for (i = 0; i < con_ptr->val_avg; i++)                            
    con_ptr->tsstak[i] = mx_amp;                                     
#else                                                                
#define bump(x) ((x) + (x)>>1) 
/*  --bump value by a factor of 1.5  */ 
	for(i = 0; i < 16; i++)                                        
		con_ptr->tsstak[i] = bump(con_ptr->tsstak[i]);         
	con_ptr->twosd = bump(con_ptr->twosd);                         
#endif                                                               

/*------------Raise the thresholds here by calling xth------------------*/
/*		after raising twosd (above)				*/


	con_ptr->th1 = Xth(con_ptr->xth1);                             
	con_ptr->th2 = Xth(con_ptr->xth2);                             
	con_ptr->th3 = Xth(con_ptr->xth3);                             
	con_ptr->thx = Xth(con_ptr->xthx);                             

/*------------To synchronize on-line and off-line monitors--------------*/
/*			See P_one for definitions			*/
						/* Note 2 */
	con_ptr->maxamp = 0;
	con_ptr->s_sum_sc = 0;
	con_ptr->sum_s_c = 0;
	con_ptr->prev_slope = 0;
	con_ptr->max_y = con_ptr->last_y = 0;
	con_ptr->tim_of_max = con_ptr->last_x = 0;
	con_ptr->index = 19;	       /*  ensure call to Ptwo expeditiously */

	con_ptr->kk = con_ptr->val_avg - 1;
	    			       /* ensure new average expeditiously */


	return;
}



