/************************************************************
* fastScript.hh                                             *
* plotting filtered time series                             *
*************************************************************
* Author:  Stefan Ballmer, MIT (sballmer@ligo.mit.edu)      *
*          Laura Cadonati, MIT (cadonati@ligo.mit.edu)      *
* Date:    Feb. 27, 2003                                    *
************************************************************/
// Enable the audio options by uncommenting the next line
// #define _USE_FASTAUDIO_

using namespace std;

#ifndef FASTGLITCH_HH
#define FASTGLITCH_HH

#include "DatEnv.hh"
#include "TSeries.hh"
#include <IIRFilter.hh>
#include <FilterBase.hh>
#include <FilterDesign.hh>
#include "Pipe.hh"
#include <string>

// EventTool header files
#include <events/Events.hh>

class fastGlitch : public DatEnv {

public:

  fastGlitch(int argc, const char *argv[]);
  ~fastGlitch(void);

  void ProcessData(void);

private:
  void SetOptions(string optstring);
  void SetNextState(void);
  void LoadConfFile(void);
  void ParseTime(string timestring);
  void ReadChannels(string Nchanstring,ifstream &file);

    char saveplot;
    char boxtype; // 0=no boxes, 1=horizontal line, 2=boxes
    char useFCQ;
    int playaudiochannel;
    char timeFromXML;
    string conffile;
    string ttfilename;

    int Nchan;           // number of channels to be analyzed
    double Tbin;          // time interval for each segment (sec)
    double SettleTime;       // time for the filter to adjust
    double TimeOffset;       // time offset (=Tbin/2 -> trigger in the middle)
    unsigned long TFirstSec; // if not 0, this is used as start time for the scan
    int Nmax;         // max number of triggers to be overlapped to the time series in Tbin segments
    int state;
    
    events::Set    trigTimes;
    events::Set::iterator trigTimes_itr;
    string         trigTimesXML;
    string         OvelayXMLFile;
    string         GraphicsPrefix;
    string         GraphicsFormat;
    string         FrameCacheQuery;
    string*        ChannelName;
    string*        FilterString;
    double*        fsample;
    FilterDesign** filterdesign; 
    Pipe**         filter; 
  
    TSeries** ts_raw; // raw time series 

};

#endif // FASTGLITCH_HH
